@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -30,8 +30,8 @@ def reject():
 
 
 def assume(condition):
-    """``assume()`` is like an :ref:`assert <python:assert>` that marks the
-    example as bad, rather than failing the test.
+    """Calling ``assume`` is like an :ref:`assert <python:assert>` that marks
+    the example as bad, rather than failing the test.
 
     This allows you to specify properties that you *assume* will be
     true, and let Hypothesis try to avoid similar examples in future.
@@ -81,7 +81,7 @@ class BuildContext(object):
         for task in self.tasks:
             try:
                 task()
-            except:
+            except BaseException:
                 any_failed = True
                 report(traceback.format_exc())
         return any_failed
@@ -123,7 +123,6 @@ def event(value):
     Events should be strings or convertible to them.
 
     """
-
     context = _current_build_context.value
     if context is None:
         raise InvalidArgument(
