@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,13 +19,43 @@ from __future__ import division, print_function, absolute_import
 
 import enum
 import math
+import heapq
+import hashlib
+from fractions import Fraction
 from collections import Sequence, OrderedDict
 
 from hypothesis._settings import note_deprecation
-from hypothesis.internal.compat import floor, hbytes, bit_length, \
-    int_from_bytes
+from hypothesis.internal.compat import floor, hbytes, hrange, qualname, \
+    bit_length, str_to_bytes, int_from_bytes
 from hypothesis.internal.floats import int_to_float
 
+LABEL_MASK = 2 ** 64 - 1
+
+
+def calc_label_from_name(name):
+    hashed = hashlib.md5(str_to_bytes(name)).digest()
+    return int_from_bytes(hashed[:8])
+
+
+def calc_label_from_cls(cls):
+    return calc_label_from_name(qualname(cls))
+
+
+def combine_labels(*labels):
+    label = 0
+    for l in labels:
+        label = (label << 1) & LABEL_MASK
+        label ^= l
+    return label
+
+
+INTEGER_RANGE_DRAW_LABEL = calc_label_from_name(
+    'another draw in integer_range()')
+GEOMETRIC_LABEL = calc_label_from_name('geometric()')
+BIASED_COIN_LABEL = calc_label_from_name('biased_coin()')
+SAMPLE_IN_SAMPLER_LABLE = calc_label_from_name('a sample() in Sampler')
+ONE_FROM_MANY_LABEL = calc_label_from_name('one more from many()')
+
 
 def integer_range(data, lower, upper, center=None):
     assert lower <= upper
@@ -54,7 +84,9 @@ def integer_range(data, lower, upper, center=None):
     probe = gap + 1
 
     while probe > gap:
+        data.start_example(INTEGER_RANGE_DRAW_LABEL)
         probe = data.draw_bits(bits)
+        data.stop_example(discard=probe > gap)
 
     if above:
         result = center + probe
@@ -72,6 +104,23 @@ def centered_integer_range(data, lower, upper, center):
 
 
 def check_sample(values):
+    try:
+        from numpy import ndarray
+        if isinstance(values, ndarray):
+            if values.ndim != 1:
+                note_deprecation((
+                    'Only one-dimensional arrays are supported for sampling, '
+                    'and the given value has {ndim} dimensions (shape '
+                    '{shape}).  This array would give samples of array slices '
+                    'instead of elements!  Use np.ravel(values) to convert '
+                    'to a one-dimensional array, or tuple(values) if you '
+                    'want to sample slices.  Sampling a multi-dimensional '
+                    'array will be an error in a future version of Hypothesis.'
+                ).format(ndim=values.ndim, shape=values.shape))
+            return tuple(values)
+    except ImportError:  # pragma: no cover
+        pass
+
     if not isinstance(values, (OrderedDict, Sequence, enum.EnumMeta)):
         note_deprecation(
             ('Cannot sample from %r, not a sequence.  ' % (values,)) +
@@ -110,8 +159,7 @@ def fractional_float(data):
 
 def geometric(data, p):
     denom = math.log1p(-p)
-
-    data.start_example()
+    data.start_example(GEOMETRIC_LABEL)
     while True:
         probe = fractional_float(data)
         if probe < 1.0:
@@ -128,7 +176,7 @@ def boolean(data):
 def biased_coin(data, p):
     """Return False with probability p (assuming a uniform generator),
     shrinking towards False."""
-    data.start_example()
+    data.start_example(BIASED_COIN_LABEL)
     while True:
         # The logic here is a bit complicated and special cased to make it
         # play better with the shrinker.
@@ -160,7 +208,11 @@ def biased_coin(data, p):
             remainder = 256 * p - truthy
 
             if falsey + truthy == 256:
-                m, n = p.as_integer_ratio()
+                if isinstance(p, Fraction):
+                    m = p.numerator
+                    n = p.denominator
+                else:
+                    m, n = p.as_integer_ratio()
                 assert n & (n - 1) == 0, n  # n is a power of 2
                 assert n > m > 0
                 truthy = m
@@ -206,66 +258,97 @@ def biased_coin(data, p):
 
 
 class Sampler(object):
-    """Sampler based on "Succinct Sampling from Discrete Distributions" by
-    Bringmann and Larsen. In general it has some advantages and disadvantages
-    compared to the more normal alias method, but its big advantage for us is
-    that it plays well with shrinking: The values are laid out in their natural
-    order, so shrink in that order.
+    """Sampler based on Vose's algorithm for the alias method. See
+    http://www.keithschwarz.com/darts-dice-coins/ for a good explanation.
 
-    Its big disadvantage is that for heavily biased distributions it can
-    use a lot of memory. Solution is some mix of "don't do that then"
-    and not worrying about it because Hypothesis is something of a
-    memory hog anyway.
+    The general idea is that we store a table of triples (base, alternate, p).
+    base. We then pick a triple uniformly at random, and choose its alternate
+    value with probability p and else choose its base value. The triples are
+    chosen so that the resulting mixture has the right distribution.
+
+    We maintain the following invariants to try to produce good shrinks:
+
+    1. The table is in lexicographic (base, alternate) order, so that choosing
+       an earlier value in the list always lowers (or at least leaves
+       unchanged) the value.
+    2. base[i] < alternate[i], so that shrinking the draw always results in
+       shrinking the chosen element.
 
     """
 
     def __init__(self, weights):
-        # We consider each weight expressed in terms of the average weight,
-        # say t. We write the weight of i as nt + f, where n is an integer and
-        # 0 <= f < 1. We then store n items for this weight which correspond
-        # to drawing i unconditionally, and if f > 0 we store an additional
-        # item that corresponds to drawing i with probability f. This ensures
-        # that (under a uniform model) we draw i with probability proportionate
-        # to its weight.
 
-        # We then rearrange things to shrink better. The table with the whole
-        # weights is kept in sorted order so that shrinking still corresponds
-        # to shrinking leftwards. The fractional weights however are put in
-        # a second table that is logically "to the right" of the whole weights
-        # and are sorted in order of decreasing acceptance probaility. This
-        # ensures that shrinking lexicographically always results in drawing
-        # less data.
-        self.table = []
-        self.extras = []
-        self.acceptance = []
-        total = sum(weights)
         n = len(weights)
-        for i, x in enumerate(weights):
-            whole_occurrences = floor(x * n / total)
-            acceptance = x - whole_occurrences
-            self.acceptance.append(acceptance)
-            for _ in range(whole_occurrences):
-                self.table.append(i)
-            if acceptance > 0:
-                self.extras.append(i)
-        self.extras.sort(key=self.acceptance.__getitem__, reverse=True)
+
+        self.table = [[i, None, None] for i in hrange(n)]
+
+        total = sum(weights)
+
+        num_type = type(total)
+
+        zero = num_type(0)
+        one = num_type(1)
+
+        small = []
+        large = []
+
+        probabilities = [w / total for w in weights]
+        scaled_probabilities = []
+
+        for i, p in enumerate(probabilities):
+            scaled = p * n
+            scaled_probabilities.append(scaled)
+            if scaled == 1:
+                self.table[i][2] = zero
+            elif scaled < 1:
+                small.append(i)
+            else:
+                large.append(i)
+        heapq.heapify(small)
+        heapq.heapify(large)
+
+        while small and large:
+            lo = heapq.heappop(small)
+            hi = heapq.heappop(large)
+
+            assert lo != hi
+            assert scaled_probabilities[hi] > one
+            assert self.table[lo][1] is None
+            self.table[lo][1] = hi
+            self.table[lo][2] = one - scaled_probabilities[lo]
+            scaled_probabilities[hi] = (
+                scaled_probabilities[hi] + scaled_probabilities[lo]) - one
+
+            if scaled_probabilities[hi] < 1:
+                heapq.heappush(small, hi)
+            elif scaled_probabilities[hi] == 1:
+                self.table[hi][2] = zero
+            else:
+                heapq.heappush(large, hi)
+        while large:
+            self.table[large.pop()][2] = zero
+        while small:
+            self.table[small.pop()][2] = zero
+
+        for entry in self.table:
+            assert entry[2] is not None
+            if entry[1] is None:
+                entry[1] = entry[0]
+            elif entry[1] < entry[0]:
+                entry[0], entry[1] = entry[1], entry[0]
+                entry[2] = one - entry[2]
+        self.table.sort()
 
     def sample(self, data):
-        while True:
-            data.start_example()
-            # We always draw the acceptance data even if we don't need it,
-            # because that way we keep the amount of data we use stable.
-            i = integer_range(data, 0, len(self.table) + len(self.extras) - 1)
-            if i < len(self.table):
-                result = self.table[i]
-                data.stop_example()
-                return result
-            else:
-                result = self.extras[i - len(self.table)]
-                accept = not biased_coin(data, 1 - self.acceptance[result])
-                data.stop_example()
-                if accept:
-                    return result
+        data.start_example(SAMPLE_IN_SAMPLER_LABLE)
+        i = integer_range(data, 0, len(self.table) - 1)
+        base, alternate, alternate_chance = self.table[i]
+        use_alternate = biased_coin(data, alternate_chance)
+        data.stop_example()
+        if use_alternate:
+            return alternate
+        else:
+            return base
 
 
 class many(object):
@@ -290,13 +373,15 @@ class many(object):
         self.rejections = 0
         self.drawn = False
         self.force_stop = False
+        self.rejected = False
 
     def more(self):
         """Should I draw another element to add to the collection?"""
         if self.drawn:
-            self.data.stop_example()
+            self.data.stop_example(discard=self.rejected)
 
         self.drawn = True
+        self.rejected = False
 
         if self.min_size == self.max_size:
             should_continue = self.count < self.min_size
@@ -312,7 +397,7 @@ class many(object):
             should_continue = biased_coin(self.data, p_continue)
 
         if should_continue:
-            self.data.start_example()
+            self.data.start_example(ONE_FROM_MANY_LABEL)
             self.count += 1
             return True
         else:
@@ -320,10 +405,11 @@ class many(object):
 
     def reject(self):
         """Reject the last example (i.e. don't count it towards our budget of
-        elements because it's not going to go in the final collection)"""
+        elements because it's not going to go in the final collection)."""
         assert self.count > 0
         self.count -= 1
         self.rejections += 1
+        self.rejected = True
         if self.rejections > 2 * self.count:
             if self.count < self.min_size:
                 self.data.mark_invalid()
