@@ -23,6 +23,7 @@ import hypothesis.internal.conjecture.utils as d
 import hypothesis.internal.conjecture.floats as flt
 from hypothesis.control import assume
 from hypothesis.internal.floats import sign
+from hypothesis.internal.conjecture.utils import calc_label_from_name
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
     MappedSearchStrategy
 
@@ -98,6 +99,9 @@ NASTY_FLOATS = sorted([
 NASTY_FLOATS = list(map(float, NASTY_FLOATS))
 NASTY_FLOATS.extend([-x for x in NASTY_FLOATS])
 
+FLOAT_STRATEGY_DO_DRAW_LABEL = calc_label_from_name(
+    'getting another float in FloatStrategy')
+
 
 class FloatStrategy(SearchStrategy):
     """Generic superclass for strategies which produce floats."""
@@ -128,7 +132,7 @@ class FloatStrategy(SearchStrategy):
 
     def do_draw(self, data):
         while True:
-            data.start_example()
+            data.start_example(FLOAT_STRATEGY_DO_DRAW_LABEL)
             i = self.sampler.sample(data)
             if i == 0:
                 result = flt.draw_float(data)
